@@ -1,12 +1,167 @@
-"""Display utilities for the CLI interface."""
+"""Display utilities for the CLI interface.
+
+All rendering occurs on the event loop thread.
+Streaming segments use Live Markdown; transient status and SQL blocks are also
+rendered with Live.
+"""
 
 import json
+from typing import Sequence, Type
 
-from rich.console import Console
-from rich.markdown import Markdown
+from pydantic_ai.messages import ModelResponsePart, TextPart
+from rich.columns import Columns
+from rich.console import Console, ConsoleOptions, RenderResult
+from rich.live import Live
+from rich.markdown import CodeBlock, Markdown
 from rich.panel import Panel
+from rich.spinner import Spinner
 from rich.syntax import Syntax
 from rich.table import Table
+from rich.text import Text
+
+
+class _SimpleCodeBlock(CodeBlock):
+    def __rich_console__(
+        self, console: Console, options: ConsoleOptions
+    ) -> RenderResult:
+        code = str(self.text).rstrip()
+        yield Syntax(
+            code,
+            self.lexer_name,
+            theme=self.theme,
+            background_color="default",
+            word_wrap=True,
+        )
+
+
+class LiveMarkdownRenderer:
+    """Handles Live markdown rendering with segment separation.
+
+    Supports different segment kinds: 'assistant', 'thinking', 'sql'.
+    Adds visible paragraph breaks between segments and renders code fences
+    with nicer formatting.
+    """
+
+    _patched_fences = False
+
+    def __init__(self, console: Console):
+        self.console = console
+        self._live: Live | None = None
+        self._status_live: Live | None = None
+        self._buffer: str = ""
+        self._current_kind: Type[ModelResponsePart] | None = None
+
+    def prepare_code_blocks(self) -> None:
+        """Patch rich Markdown fence rendering once for nicer code blocks."""
+        if LiveMarkdownRenderer._patched_fences:
+            return
+        # Guard with class check to avoid re-patching if already applied
+        if Markdown.elements.get("fence") is not _SimpleCodeBlock:
+            Markdown.elements["fence"] = _SimpleCodeBlock
+        LiveMarkdownRenderer._patched_fences = True
+
+    def ensure_segment(self, kind: Type[ModelResponsePart]) -> None:
+        """
+        Ensure a markdown Live segment is active for the given kind.
+
+        When switching kinds, end the previous segment and add a paragraph break.
+        """
+        # If a transient status is showing, clear it first (no paragraph break)
+        if self._status_live is not None:
+            self.end_status()
+        if self._live is not None and self._current_kind == kind:
+            return
+        if self._live is not None:
+            self.end()
+            self.paragraph_break()
+
+        self._start()
+        self._current_kind = kind
+
+    def append(self, text: str | None) -> None:
+        """Append text to the current markdown segment and refresh."""
+        if not text:
+            return
+        if self._live is None:
+            # default to assistant if no segment was ensured
+            self.ensure_segment(TextPart)
+
+        self._buffer += text
+        self._live.update(Markdown(self._buffer))
+
+    def end(self) -> None:
+        """Finalize and stop the current Live segment, if any."""
+        if self._live is None:
+            return
+        if self._buffer:
+            self._live.update(Markdown(self._buffer))
+        self._live.stop()
+        self._live = None
+        self._buffer = ""
+        self._current_kind = None
+
+    def end_if_active(self) -> None:
+        self.end()
+
+    def paragraph_break(self) -> None:
+        self.console.print()
+
+    def start_sql_block(self, sql: str) -> None:
+        """Render a SQL block using a transient Live markdown segment."""
+        if not sql or not isinstance(sql, str) or not sql.strip():
+            return
+        # Separate from surrounding content
+        self.end_if_active()
+        self.paragraph_break()
+        self._buffer = f"```sql\n{sql}\n```"
+        # Use context manager to auto-stop and persist final render
+        with Live(
+            Markdown(self._buffer),
+            console=self.console,
+            vertical_overflow="visible",
+            refresh_per_second=12,
+        ):
+            pass
+
+    def start_status(self, message: str = "Crunching data...") -> None:
+        """Show a transient status line with a spinner until streaming starts."""
+        if self._status_live is not None:
+            # Update existing status text
+            self._status_live.update(self._status_renderable(message))
+            return
+        live = Live(
+            self._status_renderable(message),
+            console=self.console,
+            transient=True,  # disappear when stopped
+            refresh_per_second=12,
+        )
+        self._status_live = live
+        live.start()
+
+    def end_status(self) -> None:
+        live = self._status_live
+        if live is None:
+            return
+        live.stop()
+        self._status_live = None
+
+    def _status_renderable(self, message: str):
+        spinner = Spinner("dots", style="yellow")
+        text = Text(f" {message}", style="yellow")
+        return Columns([spinner, text], expand=False)
+
+    def _start(self, initial_markdown: str = "") -> None:
+        if self._live is not None:
+            self.end()
+        self._buffer = initial_markdown or ""
+        live = Live(
+            Markdown(self._buffer),
+            console=self.console,
+            vertical_overflow="visible",
+            refresh_per_second=12,
+        )
+        self._live = live
+        live.start()
 
 
 class DisplayManager:
@@ -14,10 +169,11 @@ class DisplayManager:
 
     def __init__(self, console: Console):
         self.console = console
+        self.live = LiveMarkdownRenderer(console)
 
     def _create_table(
         self,
-        columns: list,
+        columns: Sequence[str | dict[str, str]],
         header_style: str = "bold blue",
         title: str | None = None,
     ) -> Table:
@@ -34,17 +190,24 @@ class DisplayManager:
 
     def show_tool_executing(self, tool_name: str, tool_input: dict):
         """Display tool execution details."""
-        self.console.print(f"\n[yellow]🔧 Using tool: {tool_name}[/yellow]")
+        # Normalized leading blank line before tool headers
+        self.show_newline()
         if tool_name == "list_tables":
-            self.console.print("[dim]  → Discovering available tables[/dim]")
+            self.console.print(
+                "[dim bold]:gear: Discovering available tables[/dim bold]"
+            )
         elif tool_name == "introspect_schema":
             pattern = tool_input.get("table_pattern", "all tables")
-            self.console.print(f"[dim]  → Examining schema for: {pattern}[/dim]")
+            self.console.print(
+                f"[dim bold]:gear: Examining schema for: {pattern}[/dim bold]"
+            )
         elif tool_name == "execute_sql":
+            # For streaming, we render SQL via LiveMarkdownRenderer; keep Syntax
+            # rendering for threads show/resume. Controlled by include_sql flag.
             query = tool_input.get("query", "")
-            self.console.print("\n[bold green]Executing SQL:[/bold green]")
+            self.console.print("[dim bold]:gear: Executing SQL:[/dim bold]")
             self.show_newline()
-            syntax = Syntax(query, "sql")
+            syntax = Syntax(query, "sql", background_color="default", word_wrap=True)
             self.console.print(syntax)
 
     def show_text_stream(self, text: str):
@@ -99,10 +262,12 @@ class DisplayManager:
         """Display a newline for spacing."""
         self.console.print()
 
-    def show_table_list(self, tables_data: str):
+    def show_table_list(self, tables_data: str | dict):
         """Display the results from list_tables tool."""
         try:
-            data = json.loads(tables_data)
+            data = (
+                json.loads(tables_data) if isinstance(tables_data, str) else tables_data
+            )
 
             # Handle error case
             if "error" in data:
@@ -143,10 +308,12 @@ class DisplayManager:
         except Exception as e:
             self.show_error(f"Error displaying table list: {str(e)}")
 
-    def show_schema_info(self, schema_data: str):
+    def show_schema_info(self, schema_data: str | dict):
         """Display the results from introspect_schema tool."""
         try:
-            data = json.loads(schema_data)
+            data = (
+                json.loads(schema_data) if isinstance(schema_data, str) else schema_data
+            )
 
             # Handle error case
             if "error" in data:
@@ -209,36 +376,6 @@ class DisplayManager:
         except Exception as e:
             self.show_error(f"Error displaying schema information: {str(e)}")
 
-    def show_plot(self, plot_data: dict):
-        """Display plot information and status."""
-        try:
-            # Parse the result if it's a string
-            if isinstance(plot_data.get("result"), str):
-                result = json.loads(plot_data["result"])
-            else:
-                result = plot_data.get("result", {})
-
-            # Check if there was an error
-            if "error" in result:
-                self.show_error(f"Plot error: {result['error']}")
-                return
-
-            # If plot was successful, show plot info
-            if result.get("success") and result.get("plot_rendered"):
-                plot_info = result.get("plot_info", {})
-                self.console.print(
-                    f"\n[bold green]✓ Plot rendered:[/bold green] {plot_info.get('title', 'Plot')}"
-                )
-                self.console.print(
-                    f"[dim]  Type: {plot_info.get('type', 'unknown')}, "
-                    f"Data points: {plot_info.get('data_points', 0)}[/dim]"
-                )
-
-        except json.JSONDecodeError:
-            self.show_error("Failed to parse plot result")
-        except Exception as e:
-            self.show_error(f"Error displaying plot: {str(e)}")
-
     def show_markdown_response(self, content: list):
         """Display the assistant's response as rich markdown in a panel."""
         if not content:
